@@ -1,18 +1,18 @@
-from .Base import Layer
 import cupy as cp
+
+from .Core import Layer
 from ..utils.Activator import get_activator
 
 
-
 class Activation(Layer):
-    def __init__(self,act_name='relu'):
-        self.activator=get_activator(act_name)
-        super(Activation,self).__init__()
+    def __init__(self, act_name='relu'):
+        self.activator = get_activator(act_name)
+        super(Activation, self).__init__()
 
 
     def connect(self,prev_layer):
 
-        self.output_shape=prev_layer.output_shape
+        self.output_shape = prev_layer.output_shape
         Layer.connect(self, prev_layer)
 
 
@@ -35,7 +35,7 @@ class Activation(Layer):
 
 
     def backward(self):
-        for layer in self.inbound_layers:
+        for layer in self.inbounds:
             if layer.require_grads:
                 layer.grads+=self.activator._backward(self.grads)
             else:
